﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectMachine
{
    internal class DrawningObject : IDrawningObject
    {
        public DrawningMachine _machine = null;

        public DrawningObject(DrawningMachine machine)
        {
            _machine = machine;
        }

        public float Step => _machine?.Machine?.Step ?? 0;

        public DrawningMachine GetTank => _machine;

        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return _machine?.GetCurrentPosition() ?? default;
        }

        public void MoveObject(Direction direction)
        {
            _machine?.MoveTransport(direction);
        }

        public void SetObject(int x, int y, int width, int height)
        {
            _machine.SetPosition(x, y, width, height);
        }

        void IDrawningObject.DrawningObject(Graphics g)
        {
            _machine.DrawTransport(g);
        }

        public string GetInfo() => _machine?.GetDataForSave();

        public static IDrawningObject Create(string data) => new DrawningObject(data.CreateDrawningMachine());

        public bool Equals(IDrawningObject? other)
        {
            if (other == null)
            {
                return false;
            }
            var otherMachine = other as DrawningObject;
            if (otherMachine == null)
            {
                return false;
            }
            var machine = _machine.Machine;
            var otherMachineMachine = otherMachine._machine.Machine;
            if (machine.Speed != otherMachineMachine.Speed)
            {
                return false;
            }
            if (machine.Weight != otherMachineMachine.Weight)
            {
                return false;
            }
            if (machine.BodyColor != otherMachineMachine.BodyColor)
            {
                return false;
            }
            //проверка на то, что являются ли оба объекта продвинутыми
            if (machine is EntityTank && otherMachineMachine is not EntityTank)
            {
                return false;
            }
            if (machine is not EntityTank && otherMachineMachine is EntityTank)
            {
                return false;
            }
            //если оба объекта являются продвинутыми, сравниваем дополнительные характеристики
            if (machine is EntityTank tank && otherMachineMachine is EntityTank otherTank)
            {
                if (tank.DopColor != otherTank.DopColor)
                {
                    return false;
                }
                if (tank.Gun != otherTank.Gun)
                {
                    return false;
                }
                if (tank.Turret != otherTank.Turret)
                {
                    return false;
                }
            }
            return true;
        }

    }
}
